package cam.visuals.contours;
import java.awt.*;
//
//###################################################################
//                   Class ColorContourCanvas
//###################################################################
//
//
//  Creator               : Chris Anderson
//  (C) UCLA 1997
//
//###################################################################
/**
This class creates a color contour for a two dimensional (rectangular) data set. 
The color contour is created by coloring squares of a grid with colors that are
associated with the data set function values. The grid used for the 
contour need not match the size of the data set - the data set is bi-linearly
interpolated to get any needed function values. 
*/
public class ColorContourCanvas extends java.awt.Canvas
{
public ColorContourCanvas()
{
    super();
    //Parent      = parent;

    xDataCount = 0;
    yDataCount = 0;

    xCanvasPanelCount = 60;
    yCanvasPanelCount = 60;

    contourData  = null;
    dataRangeMin = 0.0;
    dataRangeMax = 0.0;

    colorCount  = 64;
    colorArray = new Color[colorCount];
    setupColors();
}
/**
Sets the data to be contoured. 
@param inputData one dimensional array of doubles containing the two dimensional data set 
values. It is assumed that the two dimensional data is stored by <it>rows</it>
@param xCount the number of data values in the x direction
@param yCount the number of data values in the y direction 
*/
public void setData(double [] inputData, int xCount, int yCount)
{
//
//  This routine assumes that the incoming data is stored by ROWS
//
    contourData = new double[xCount*yCount];
    System.arraycopy(inputData,0,contourData,0,xCount*yCount);
    xDataCount = xCount;
    yDataCount = yCount;
    repaint();
}
/**
Sets the data to be contoured. 
@param inputData two dimensional array of doubles containing the data set 
values.
*/
public void setData(double [][] inputData)
{
    xDataCount = inputData.length;
    yDataCount = inputData[0].length;
    contourData = new double[xDataCount*yDataCount];
    int i;
    for(i = 0; i < xDataCount; i++)
    {
    System.arraycopy(inputData[i],0,contourData,i*yDataCount,yDataCount);
    }
    repaint();
}
/**
Sets the maximal and minimal values of the data range that will be contoured.
@param dMin the minimum data set value to be contoured
@param dMax the maximum data set value to be contoured
*/
public void setDataRange(double dMin, double dMax)
{
    dataRangeMin = dMin;
    dataRangeMax = dMax;
}
/**
Sets the resolution of the color contour plot
@param PanelCount the number of panels in the x and y directions to be used for
the color contour grid.
*/
public void setResolution(int PanelCount)
{
    xCanvasPanelCount = PanelCount;
    yCanvasPanelCount = PanelCount;
}
/**
Returns the number of panels of the color contour grid.
*/
public int  getResolution()
{
    return xCanvasPanelCount;
}
/**
Determines and then sets the data range to be the  maximal and minimal values of 
contour data.
*/
public void setDataRange()
{
    if(contourData != null)
    {
    dataRangeMin = contourData[0];
    dataRangeMax = contourData[0];
    double val;
    int i; int j;
    for(i =0;  i < xDataCount; i++)
    {
    for(j = 0; j < yDataCount; j++)
    {
       val = contourData[j + i*yDataCount];
       if(val < dataRangeMin) dataRangeMin = val;
       if(val > dataRangeMax) dataRangeMax = val;
    }}
    }
}
public Dimension getPreferredSize(){
      return new Dimension(300,300);
   }
public Dimension getMinimumSize(){
      return getPreferredSize();
   }
//
// Using bilinear interpolation of the input data; Data is
// assumed to be at the nodes of a grid. The rectangle color value
// is taken to be the value at the center of the rectangle of
// a "canvas" grid decomposed into xCanvasPanelCount X yCanvasPanelCount
// panels.
//
public void paint(Graphics g)
{
    if(contourData == null) return;

    if(Math.abs(dataRangeMax- dataRangeMin) < 1.0e-10) setDataRange();
    if(Math.abs(dataRangeMax- dataRangeMin) < 1.0e-10) return;
    Dimension D = size();

    int boxWidth  = D.width/xCanvasPanelCount;
    int boxHeight = D.height/yCanvasPanelCount;
    if(boxWidth  <= 1)
    {
     xCanvasPanelCount = D.width/2;
     boxWidth  =  D.width/xCanvasPanelCount;
    }
    if(boxHeight <= 1)
    {
    yCanvasPanelCount = D.height/2;
    boxHeight = D.height/yCanvasPanelCount;
    }
    int xOffset = (D.width  - xCanvasPanelCount*boxWidth)/2;
    int yOffset = (D.height - yCanvasPanelCount*boxHeight)/2;
    int xCanvasLocation;
    int yCanvasLocation;


    double canvasHx; double canvasHy;
    double dataHx;   double dataHy;
    canvasHx = 1.0/((double)(xCanvasPanelCount));
    canvasHy = 1.0/((double)(yCanvasPanelCount));
    dataHx   = 1.0/((double)(xDataCount-1));
    dataHy   = 1.0/((double)(yDataCount-1));

    double xLocation;
    double yLocation;

    int xDataIndex;
    int yDataIndex;

    int i; int j;
    double colorLocation;
    double dataValue;
    int colorIndex;

    double p; double q;
    double v1; double v2; double v3; double v4;
    double phi0; double phi1;

    for(i =0; i < xCanvasPanelCount; i++)
    {
    xCanvasLocation  = xOffset + i*boxWidth;
    xLocation        = canvasHx/2.0 + ((double)i)*canvasHx;
    xDataIndex = (int)(xLocation/dataHx);
    for(j = 0; j < yCanvasPanelCount; j++)
    {
    yCanvasLocation  = yOffset + j*boxHeight;
    yLocation        = canvasHy/2.0 + ((double)j)*canvasHy;
    yDataIndex = (int)(yLocation/dataHy);
    //
    // do bilinear interpolation
    //
    p = (xLocation - xDataIndex*dataHx)/dataHx;
    q = (yLocation - yDataIndex*dataHy)/dataHy;
    v1 = contourData[yDataIndex + xDataIndex*yDataCount];
    v2 = contourData[yDataIndex+ (xDataIndex+1)*yDataCount];
    v3 = contourData[(yDataIndex + 1) + (xDataIndex+1)*yDataCount];
    v4 = contourData[(yDataIndex + 1) + xDataIndex*yDataCount];
    phi0 = v1 + p*(v2 - v1);
    phi1 = v4 + p*(v3 - v4);
    dataValue = phi0 + q*(phi1 - phi0);

    colorLocation =
    ((colorCount-1.0)*(dataValue - dataRangeMin))/(dataRangeMax - dataRangeMin);
    colorIndex = (int)colorLocation;
    if(colorIndex >= colorCount) colorIndex = colorCount-1;
    if(colorIndex < 0) colorIndex = 0;
    g.setColor(colorArray[colorIndex]);
    g.fillRect(xCanvasLocation,yCanvasLocation,boxWidth,boxHeight);
    }}
    g.setColor(Color.black);
}
//
//###################################################################
//                  SETTING UP COLOR TABLE
//###################################################################
//
/**
Sets up color table with values from the r,g and b arrays.
@param r,g,b double arrays with values in the range [0.0, 1.0] specifying the red,
green and blue components of the colors to be used in the contour. 
*/
public void setupColorTable(double [] r, double [] g, double [] b)
{
//
//  Setting up colors based color table value in the double
//  arrays r g and b.
//
    colorCount = r.length;
    colorArray = new Color[colorCount];
    int i;
    for(i = 0; i < colorCount; i++)
    {
        colorArray[i] = new Color((float)r[i], (float)g[i], (float)b[i]);
    }
}
/**
Returns the number of colors used in the color table.
*/
public int getColorCount() {return colorCount;}

/**
Sets the number of colors used in the color table. 
*/
public void setColorCount(int cCount)
{
    int i;
    Color [] oldColors  = colorArray;
    colorArray = new Color[cCount];
    if(colorCount <= cCount)
    {
       for(i = 0; i < colorCount; i++) colorArray[i] = oldColors[i];
       for(i = colorCount; i < cCount; i++) colorArray[i] = new Color(0,0,0);
    }
    else
    {
      for(i = 0; i < cCount; i++) colorArray[i] = oldColors[i];
    }
    colorCount = cCount;
}
/**
Sets a specific color table entry.
*/
public void setColorTableEntry(int index, double r, double g, double b)
{
    colorArray[index] = new Color((float)r,(float)g,(float)b);
}
/**
Sets up a default color table.
*/
public void setupColors()
{
//
//  Setting up colors based upon hue....
//
    int i;
    float hinc = (float)(1.0/colorCount);
    float h = hinc;
    float saturation = (float)(0.8);
    float intensity  = (float)(0.9);
    for(i = 0; i < colorCount; i++)
    {
        int color = Color.HSBtoRGB(h, saturation, intensity);
        h +=  hinc;
        colorArray[i] = new Color(color);
    }
}
    //Container Parent;

    int xCanvasPanelCount;  // determines the resolution of the canvas
    int yCanvasPanelCount;

    int xDataCount;
    int yDataCount;

    double [] contourData;
    double dataRangeMin;
    double dataRangeMax;

    int colorCount;
    Color[] colorArray;

}