package cam.visuals.plots;
//***************************************************************
//				File:  Plot.java
//				Creator :  by Mark Hoefer
//
//              Modifications : 
//              10/18/98 - Chris Anderson -
//              Migration to Java 1.1 event model + bug fixes
//
//				(C) UCLA 1997,1998
//***************************************************************

import java.awt.*;
import java.io.*;
import java.util.Vector;
/**
An object which takes x and y data arrays and plots them.
Various attributes of the plot can be set including axes type,
grid lines, labels, and title. This class allows multiple plots to be
"overlaid", each separate plot inheriting the current bounding box
and labelling. The individual plots are accessed via a plot name.
@author Mark Hoefer, Chris Anderson
@version Oct. 18, 1998
*/
public class Plot extends Canvas
{
//
//***************************************************************
//                           METHODS
//***************************************************************
/**
Creates a Plot instance and sets the bounding rectangle and defaults.
*/
public Plot()
{
		setDefaults();	
		SymMouse aSymMouse = new SymMouse();	
        this.addMouseListener(aSymMouse);
        SymMouseMotion aSymMouseMotion = new SymMouseMotion();
		this.addMouseMotionListener(aSymMouseMotion);
}
//
// ACCESS METHODS
//
/**
Returns the distance between the X coordinate tick marks.
*/
public double getXTickScale(){ return xTickScale; }
/**
Returns the distance between the Y coordinate tick marks.
*/
public double getYTickScale(){ return yTickScale; }
/**
Returns the minimum X coordinate of the plot bounding box.
*/
public double getXMin(){ return xmin; }
/**
Returns the minimum Y coordinate of the plot bounding box.
*/
public double getYMin(){ return ymin; }
/**
Returns the maximum X coordinate of the plot bounding box.
*/
public double getXMax(){ return xmax; }
/**
Returns the maximum Y coordinate of the plot bounding box.
*/
public double getYMax(){ return ymax; }
/**
Returns the current plot color.
@param plotName the name of the plot whose color is desired.
*/
public Color getPlotColor(String plotName)
{
		PlotData pData = findPlotData(plotName);
		if(pData == null) return null;
		return pData.getColor();
}
/**
Returns the current plot character.
@param plotName the name of the plot whose plot character is desired.
*/
public char getPlotChar(String plotName)
{
		PlotData pData = findPlotData(plotName);
		if(pData == null)
			return '*';
		return pData.getPointChar();
}
/**
Returns the current plot type (e.g. Plot.CURVE, Plot.CURVE_AND_POINTS, etc).
@param plotName the name of the plot whose plot character is desired.
*/
public int getPlotType(String plotName)
{
		PlotData pData = findPlotData(plotName);
		if(pData == null) return -1;
		return pData.getPlotType();
}
/**
Returns a string array containing the names of the plots.
*/
public String[] getPlotNames()
{
		int num = data.size();  // the size of the Vector of PlotData's
		String[] names = new String[num];
		for(int i=0;i<num;i++)
			names[i] = ((PlotData)data.elementAt(i)).getPlotName();
		return names;
}
//
// MUTATOR METHODS
//
/**
Changes the x and y data of a plot destructively.
@param plotName the name of the plot targeted for a data change
@param xdata x-component of the data to be plotted
@param ydata y-component of the data to be plotted
@return true if successful.
*/
public boolean changeData(String plotName, double[] xdata, double[] ydata)
	{
		PlotData pData = findPlotData(plotName);
		if(pData == null)
			return false;
		pData.setData(xdata,ydata);
		repaint();
		return true;
	}
/**
Sets the plot type.
@param plotName the name of the target plot
@param type one of Plot.CURVE, Plot.POINTS, Plot.CURVE_AND_POINTS
*/
	public boolean setPlotType(String plotName, int type)
	{
		PlotData pData = findPlotData(plotName);
		if(pData == null)
			return false;
		pData.setPlotType(type);
		return true;
	}
//
/**
Changes the name of a plot
@param plotName target plot's name
@param newPlotName new name for target plot
@return  true if successful
*/
public boolean setPlotName(String plotName, String newPlotName)
{
		PlotData pData = findPlotData(plotName);
		if(pData == null)
			return false;
		pData.setPlotName(newPlotName);
		return true;
	}
/**
Sets the plot color. 
@param plotName name of the target plot
@param c color
*/
	public boolean setPlotColor(String plotName, Color c)
	{
		PlotData pData = findPlotData(plotName);
		if(pData == null)
			return false;
		pData.setColor(c);
		return true;
	}
/**
Set the character constant used for plotting
@param plotName name of the target plot
@param ch plotting character to be used
@return true if successful
*/
	public boolean setPlotChar(String plotName, char ch)
	{
		PlotData pData = findPlotData(plotName);
		if(pData == null)
			return false;
		pData.setPlotChar(ch);
		return true;
	}
/**
Sets color and font for the drawing of the boundary points
*/
	public void setBoundPointsInfo(Font f, Color c)
	{
		boundPointsFont = f;
		boundPointsColor = c;
	}
/**
Sets tick scale. The tick scale is the distance between tick marks
@param xScale : distance between the x-tick marks
@param yScale : distance between the y-tick marks
*/
	public void setTickScale(double xScale, double yScale)
	{
		xTickScale = xScale;
		yTickScale = yScale;
		autoTickScaling = false;
	}
/**
Sets the x coordinate label precision
@param precision number of digits used in the label
*/
	public void setXlabelPrecision(int precision)
	{
	   xPrecision = precision;
	}
/**
Sets the y coordinate label precision
@param precision number of digits used in the label
*/
	public void setYlabelPrecision(int precision)
	{
	   yPrecision = precision;
	}
/**
Sets the label format for the x coordinate. 
@param type one of Plot.FIXED or Plot.SCIENTIFIC
*/
	public void setXlabelFormat(int type)
	{
	  if(type == FIXED)xLabelScientific = false;
	  else xLabelScientific = true;
	}
/**
Sets the label format for the y coordinate. 
@param type one of Plot.FIXED or Plot.SCIENTIFIC
*/
	public void setYlabelFormat(int type)
	{
	  if(type == FIXED)yLabelScientific = false;
	  else yLabelScientific = true;
	}
/**
Sets the coordinate bounds for the plot. 
@param xmin Minimum x coordinate 
@param xmax Maximum y coordinate
@param ymin Minimum y coordinate
@param ymax Maximum y coordinate
*/
public void setXYBounds(double xmin, double xmax, double ymin,  double ymax)
{
		this.xmin = xmin;
		this.ymin = ymin;
		this.xmax = xmax;
		this.ymax = ymax;

		setAutoScaling(false);

		if(autoTickScaling)
		{
			//
			// Default number of tick marks is 5
			//
			xTickScale = Math.abs(xmax-xmin)/5;
			yTickScale = Math.abs(ymax-ymin)/5;
		}
}
/**
Sets the minimum and maximum x coordinate of the bounding box.
@param xmin the minimum x coordinate of the bounding box
@param xmax the maximum x coordinate of the bounding box
*/
public void setXbounds(double xmin, double xmax)
{
		this.xmin = xmin;
		this.xmax = xmax;
		
		if(autoTickScaling)
		{xTickScale = Math.abs(xmax-xmin)/5;}
}
/**
Sets the minimum and maximum y coordinate of the bounding box.
@param ymin the minimum y coordinate of the bounding box
@param ymax the maximum y coordinate of the bounding box
*/
public void setYbounds(double Ymin, double Ymax)
{
	this.ymin = Ymin;
	this.ymax = Ymax;
    if(autoTickScaling){yTickScale = Math.abs(ymax-ymin)/5;}
}
/**
Sets the maximum y coordinate of the default bounding box.
@param yMaxDef the maximum y coordinate of the bounding box
*/
public void setYmaxDefault(double yMaxDef)
{
	    yMaxDefault = yMaxDef;
}
/**
Sets the minimum y coordinate of the default bounding box.
@param yMinDef the minimum y coordinate of the bounding box
*/
public void setYminDefault(double yMinDef)
{
	    yMinDefault = yMinDef;
}
/**
Seets the title of the plot
@param t the title
@param f the font 
@param c the color c
*/
public void setTitle(String t, Font f, Color c)
{
		title = t;
		titleFont = f;
		titleColor = c;
}
/**
Removes the title from the plot.
*/
public void removeTitle()
{
		title = null;
}
/**
Sets the type of axis used. 
@param type Axis type, one of Plot.NONE, 
Plot.PERIMETER, Plot.FLOATING, Plot.XAXIS
*/
public void setAxisType(int type)
{
	if(type != NONE && type != PERIMETER && type != FLOATING && type != XAXIS)
	{
		System.err.println("Invalid axis type.");
		return;
	}
	axisType = type;
}
/**
Sets the axis labels. 
@param xLabel the x axis label
@param yLabel the y axis label
@param f the label font
@param c the label color
*/
public void setAxisLabels(String xLabel, String yLabel,
									  Font f, Color c)
{
		xAxisLabel = xLabel;
		yAxisLabel = yLabel;
		axisLabelFont = f;
		axisLabelColor = c;
		drawAxisLabels = true;
}
/**
This function zooms in the x and y directions of the bounding box
by the given percentage. >100 will zoom out, <100 will zoom in
*/
public void zoom(double xPercentage,double yPercentage)
{
		setXYBounds(.5*(xmax*(1-xPercentage/100.0)+xmin*(1+xPercentage/100.0)),
		             .5*(xmax*(1+xPercentage/100.0)+xmin*(1-xPercentage/100.0)),
					 .5*(ymax*(1-yPercentage/100.0)+ymin*(1+yPercentage/100.0)),
					 .5*(ymax*(1+yPercentage/100.0)+ymin*(1-yPercentage/100.0)));
}
//
// PLOTTING METHODS
//
/**
Plots the data with specified color, character and type. 
@param xdata the x coordinates of the data
@param ydata the y coordinates of the data
@param c the color used to plot the data
@param ch the character used to indicate data points
@param plotName the name of the plot associated with the data
@param type the type of plot. One of Plot.CURVE, Plot.POINTS, Plot.CURVE_AND_POINTS
*/
public void plot(double[] xdata, double[] ydata, Color c, char ch,
						  String plotName, int type)
{
	//
	// Try to remove the plot if already exists
	//
		removePlot(plotName);
		data.addElement(new PlotData(xdata,ydata,c,ch,plotName,type));
		//
		// Automatically generate bounding box if auto scaling is on
		//
	    int i;
	    
	    double maxX;
		double maxY;
		double minX;
		double minY;
			
		if((autoScale)&&(xdata.length > 0))
		{
		  
			maxX = xdata[0];
			maxY = ydata[0];
			minX = xdata[0];
			minY = ydata[0];

			//
			// Loop through data, finding max and minimum
			//

			for(i=0;i<xdata.length;i++)
			{
				if(xdata[i] >= maxX)
					maxX = xdata[i];
				else if(xdata[i] < minX)
					minX = xdata[i];
				if(ydata[i] >= maxY)
					maxY = ydata[i];
				else if(ydata[i] < minY)
					minY = ydata[i];
			}
			
			maxY  = Math.min(yMaxDefault,maxY);
			minY  = Math.max(yMinDefault,minY);
			
			if(axisType == XAXIS)
			{
			    maxY = Math.max(Math.abs(maxY),Math.abs(minY));
			    minY = - maxY;
			}
		//
		// Add 1/6 to the top and bottom
		//
			double heightChange = Math.abs((maxY-minY)/6.0);
			minY -= heightChange;
			maxY += heightChange;
			
			xmax = Math.max(maxX, xmax);
			ymax = Math.max(maxY, ymax);
			xmin = Math.min(minX, xmin);
			ymin = Math.min(minY, ymin);
		}
		else
		{
		maxX = xdata[0];
	    minX = xdata[0];
		for(i=0;i<xdata.length;i++)
		{
		    if(xdata[i] >= maxX)
			maxX = xdata[i];
			else if(xdata[i] < minX)
			minX = xdata[i];
	    }   
			xmax = Math.max(maxX, xmax);
			xmin = Math.min(minX, xmin);
		}
		if(autoTickScaling)
		{
			//
			// Default number of tick marks is 5
			//
			xTickScale = Math.abs(xmax-xmin)/5;
			yTickScale = Math.abs(ymax-ymin)/5;
		}
		repaint();
	}
/**
Plots the data and assocates a name with the plot. 
@param xdata the x coordinates of the data
@param ydata the y coordinates of the data
@param plotName the name of the plot associated with the data
*/
public void plot(double[] xdata, double[] ydata, String plotName)
{
		plot(xdata,ydata,Color.black,'*',plotName,Plot.CURVE);
}
/**
Plots the data with specified color and associates a name with the plot.
@param xdata the x coordinates of the data
@param ydata the y coordinates of the data
@param c the color used to plot the data
@param plotName the name of the plot associated with the data
*/
public void plot(double[] xdata, double[] ydata, Color c, String plotName)
{
		plot(xdata,ydata,c,'*',plotName,Plot.CURVE);
}
/**
Plots the y coordinates of the data with respect to their index, and associates a name
with the plot.
@param ydata the y coordinates of the data
@param plotName the name of the plot associated with the data
*/
public void plot(double[] ydata, String plotName)
{
		plot(ydata,Color.black,'*',plotName,Plot.CURVE);
}
/**
Plots the y coordinates of the data with respect to their index using charater
markers and associates a name
with the plot. 
@param ydata the y coordinates of the data
@param ch the character used to indicate data points
@param plotName the name of the plot associated with the data
*/
public void plot(double[] ydata, char ch, String plotName)
{
		plot(ydata,Color.black,ch,plotName,Plot.POINTS);
}
/**
Plots the y coordinates of the data with respect to their index using charater
markers and a specified color. 
@param xdata the x coordinates of the data
@param ydata the y coordinates of the data
@param c the color used to plot the data
@param ch the character used to indicate data points
@param plotName the name of the plot associated with the data
@param type the type of plot. One of Plot.CURVE, Plot.POINTS, Plot.CURVE_AND_POINTS
*/
public void plot(double[] ydata, Color c, char ch, String plotName, int type)
{
		double[] xdata = new double[ydata.length];
		for(int i=0;i<xdata.length;i++)
			xdata[i] = i;
		plot(xdata,ydata,c,ch,plotName,type);
}
/**
Removes the named plot.
@param plotName the target plot to be removed.
*/
public boolean removePlot(String plotName)
{
		PlotData pData = findPlotData(plotName);
		if(pData == null)
			return false;
		data.removeElement(pData);
		return true;
	}
/**
Deletes all plots and resets defaults
*/
public void clear()
{
		data.removeAllElements();
		setDefaults();
}
//
// FLAG ACCESSOR METHODS
//
public boolean isZoomRectangleOn(){ return drawZoom; }
public boolean isGridLinesOn(){ return drawGridLines;	}
public boolean isBoundPointsOn(){ return drawBoundPoints; }
public boolean isTickMarksOn(){ return drawTicks; }
public boolean isAutoScalingOn(){ return autoScale; }
public boolean isAxisLabelsOn(){ return drawAxisLabels; }
public boolean isAutoTickScalingOn(){ return autoTickScaling; }
//
// FLAG MUTATOR METHODS
//
public void setAutoTickScaling(boolean state)
{
		autoTickScaling = state;
		// Call setXYBounds() so that tick marks are redrawn
		setXYBounds(xmin,xmax,ymin,ymax);
}
	
public void setDrawZoomRectangle(boolean state)
{
		drawZoom = state;
		
		if(!drawZoom)
		{
			Graphics g = getGraphics();
			if(g == null)
				return;
			g.setXORMode(new Color(64,64,64));
		//
		// Draw over last drawn rectangle
		//
			g.drawLine(firstX,firstY,firstX,lastY);
			g.drawLine(firstX,firstY,lastX,firstY);
			g.drawLine(lastX,firstY,lastX,lastY);
			g.drawLine(firstX,lastY,lastX,lastY);
			g.dispose();
		}
		firstX = lastX = firstY = lastY = 0;
}
public void setGridLines(boolean state)
{
		drawGridLines = state;
}
public void setDrawBoundPoints(boolean state)
{
		drawBoundPoints = state;
}
public void setDrawTickMarks(boolean state)
{
		drawTicks = state;
}
public void setDrawAxisLabels(boolean state)
{
		drawAxisLabels = state;
}
public void setAutoScaling(boolean state){ autoScale = state; }


//
// PAINTING METHODS
//
/**
Induces a repainting of the plot. Used to cause a Plot object
repaint after a Plot parameter change.
*/	
public void rePlot()
{
	   validate();
	   repaint();
}
/**
Paints the plot into the input Graphics object
@param g input Graphics object
*/
public void paint(Graphics g)
{		
	    pGraphics = g;
		setOffsets();
		
		paintAxes();
		if(data.isEmpty())  // bail out if nothing to draw yet
			return;

		paintAxisLabels();
		paintTitle();
		paintTickMarks();
		paintGridLines();
		paintBoundPoints();
		for(int i=0;i<data.size();i++)
	    {paintPlot((PlotData)data.elementAt(i));}
		// make sure that a new rectangle isn't painted
		firstX = lastX = firstY = lastY = 0;
        pGraphics = null;
}
private void paintPlot(PlotData P)
{
	    if(pGraphics == null) return;
		Graphics g = pGraphics;
		g.setColor(P.getColor());
		g.setFont(new Font("TimesNewRoman",Font.PLAIN,10));
		g.clipRect(leftOffset,topOffset,getSize().width-leftOffset-rightOffset,
				  getSize().height-topOffset-bottomOffset);
		double[] x = P.getXdata();
		double[] y = P.getYdata();
		char[] ch = new char[1];
		ch[0] = P.getPointChar();
		FontMetrics fm = g.getFontMetrics(g.getFont());
		// make sure that points aren't too far out of drawing area
		// avoids crazy lines that result from zooming in too close
		Rectangle bounds = new Rectangle(xUserToScreen(xmin),yUserToScreen(ymax),
		xUserToScreen(xmax)-xUserToScreen(xmin),
		yUserToScreen(ymin)-yUserToScreen(ymax));
		int xnow, ynow, xnext, ynext;

		switch(P.getPlotType())
		{
			case Plot.CURVE:

				for(int j=0;j<x.length-1;j++)
				{
					xnow = xUserToScreen(x[j]);
					ynow = yUserToScreen(y[j]);
					xnext = xUserToScreen(x[j+1]);
					ynext = yUserToScreen(y[j+1]);
					
					//Check if at least one point is inside drawing area
					if(bounds.contains(xnow,ynow) || bounds.contains(xnext,ynext))
					g.drawLine(xnow+leftOffset,ynow+topOffset,
							  xnext+leftOffset,ynext+topOffset);
				}

				break;

			case Plot.POINTS:

				for(int j=0;j<x.length;j++)
					g.drawChars(ch,0,1,xUserToScreen(x[j])+leftOffset-
											 (int)(fm.charWidth(ch[0])/2),
							  	  			 yUserToScreen(y[j])+topOffset+
							  	  			 (int)(fm.getAscent()/2));
				break;

			case Plot.CURVE_AND_POINTS:

				//
				// Need to fiddle with FontMetrics in order to place
				// the point char correctly centered
				//
				for(int j=0;j<x.length-1;j++)
				{
					xnow = xUserToScreen(x[j]);
					ynow = yUserToScreen(y[j]);
					xnext = xUserToScreen(x[j+1]);
					ynext = yUserToScreen(y[j+1]);
					
					// Check if at least one point is inside drawing area
					if(bounds.contains(xnow,ynow) || bounds.contains(xnext,ynext))
						g.drawLine(xnow+leftOffset,ynow+topOffset,
									  xnext+leftOffset,ynext+topOffset);
					g.drawChars(ch,0,1,xUserToScreen(x[j])+leftOffset-
											 (int)(fm.charWidth(ch[0])/2),
							  	  			 yUserToScreen(y[j])+topOffset+
							  	  			 (int)(fm.getAscent()/2));
				}
				g.drawChars(ch,0,1,xUserToScreen(x[x.length-1])+leftOffset-
											 (int)(fm.charWidth(ch[0])/2),
							  	  			 yUserToScreen(y[y.length-1])+topOffset+
							  	  			 (int)(fm.getAscent()/2));
				break;
		}
	}
   private void paintAxisLabels()
   {
   	if(!drawAxisLabels)   return;
   	if(pGraphics == null) return;
   	Graphics g = pGraphics;
   	g.setFont(axisLabelFont);
   	g.setColor(axisLabelColor);
   	FontMetrics fm = g.getFontMetrics(g.getFont());

   	g.drawString(xAxisLabel,
   					 getSize().width/2-fm.stringWidth(xAxisLabel)/2+leftOffset/2,
   					 getSize().height-8);
   	int startHeight = (getSize().height-topOffset-bottomOffset)/2-
   							(yAxisLabel.length()/2-1)*fm.getHeight();
   	paintVerticalText(g,yAxisLabel,5,startHeight);
	}
//
// This function draws text vertically on a Graphics object
// startX, startY is the baseline point of the first character in text
//
	private void paintVerticalText(Graphics g, String text,
											int startX, int startY)
	{
		int currentY = startY;
		int currentLetter;
		char[] chs = new char[1];
		// use this stream to get one char at a time
		StringBufferInputStream is = new StringBufferInputStream(text);
		FontMetrics fm = g.getFontMetrics(g.getFont());
		// allows us to find the width of each character
		// used in centering
		int[] charWidths = fm.getWidths();
		// draw each letter in a column
		is.reset();
		while((currentLetter = is.read()) != -1)
		{
			chs[0] = (char)currentLetter;
			g.drawChars(chs,0,1,
							startX+(fm.getMaxAdvance()-charWidths[currentLetter])/2,
							currentY+fm.getAscent());
			currentY += fm.getHeight();
		}
	}
	
	private void paintBoundPoints()
	{
		if(!drawBoundPoints) return;
		if(pGraphics == null) return;

		Graphics g = pGraphics;
		g.setColor(Color.black);
		g.setFont(new Font("TimesNewRoman",Font.PLAIN,12));
		FontMetrics fm = g.getFontMetrics(g.getFont());

		String s1 = "("+NtoS.toString(xmin)+","+ NtoS.toString(ymin)+")";
		String s2 = "("+NtoS.toString(xmax)+","+ NtoS.toString(ymax)+")";
		g.drawString(s1,3,getSize().height-8);
		g.drawString(s2,getSize().width-fm.stringWidth(s2)-3,fm.getHeight());
	}
	
	
	private void paintGridLines()
	{
	if(!drawGridLines)return;
	if(pGraphics == null) return;
	

	Graphics g = pGraphics;
	g.setColor(Color.gray);
    double currentX;
    double currentY;

   //
   // Start at origin of axes and draw tick marks from there
	// using the tickScale
   //
      if(axisType == PERIMETER)
      {
      	currentX = xmin + xTickScale;
      	while(currentX < xmax)
      	{
      		g.drawLine(xUserToScreen(currentX)+leftOffset,topOffset,
						  	  xUserToScreen(currentX)+leftOffset,
						  	  getSize().height-bottomOffset);
				currentX += xTickScale;
      	}

      	currentY = ymin + yTickScale;
      	while(currentY < ymax)
      	{
      		g.drawLine(leftOffset,yUserToScreen(currentY)+topOffset,
      					  getSize().width-rightOffset,
							  yUserToScreen(currentY)+topOffset);
				currentY += yTickScale;
      	}
      }

      else if(axisType == FLOATING || axisType == NONE)
      {
      	double maxX = Math.max(Math.abs(xmin),Math.abs(xmax));
      	currentX = 0;
      	while(currentX < maxX)
			{
				g.drawLine(xUserToScreen(currentX)+leftOffset,topOffset,
							  xUserToScreen(currentX)+leftOffset,
							  getSize().height-bottomOffset);
				g.drawLine(xUserToScreen(-currentX)+leftOffset,topOffset,
							  xUserToScreen(-currentX)+leftOffset,
							  getSize().height-bottomOffset);
				currentX += xTickScale;
			}

			double maxY = Math.max(Math.abs(ymin),Math.abs(ymax));
			currentY = 0;
			while(currentY < maxY)
			{
				g.drawLine(leftOffset,yUserToScreen(currentY)+topOffset,
							  getSize().width-rightOffset,
							  yUserToScreen(currentY)+topOffset);
				g.drawLine(leftOffset,yUserToScreen(-currentY)+topOffset,
							  getSize().width-rightOffset,
							  yUserToScreen(-currentY)+topOffset);
				currentY += yTickScale;
			}
      }
   }
	private void paintTickMarks()
	{
		if(!drawTicks || axisType == NONE) return;
		if(pGraphics == null) return;
		
		Graphics g = pGraphics;
		g.setColor(Color.black);
		g.setFont(new Font("TimesNewRoman",Font.PLAIN,10));
		FontMetrics fm = g.getFontMetrics(g.getFont());
	//
	// Start at origin of axes and draw tick marks from there
	// using the tickScale
	//
		if(axisType == FLOATING)
		{
			//
			// keep tick marks in drawing area
			//
			g.clipRect(leftOffset,topOffset,getSize().width-leftOffset-rightOffset,
					  getSize().height-topOffset-bottomOffset);
			double currentX = xTickScale;
			double maxX = Math.max(Math.abs(xmin),Math.abs(xmax));
			String sPos, sNeg;
			while(currentX < maxX)
			{
				sPos = numToString(currentX,0);
				sNeg = numToString(-currentX,0);
				g.drawLine(xUserToScreen(currentX)+leftOffset,
							  yUserToScreen(0.0)+5+topOffset,
							  xUserToScreen(currentX)+leftOffset,
							  yUserToScreen(0.0)+topOffset-4);
				g.drawString(sPos,xUserToScreen(currentX)+leftOffset-
										fm.stringWidth(sPos)/2,
										yUserToScreen(0.0)+3+topOffset+fm.getHeight());
				g.drawLine(xUserToScreen(-currentX)+leftOffset,
							  yUserToScreen(0.0)+5+topOffset,
							  xUserToScreen(-currentX)+leftOffset,
							  yUserToScreen(0.0)+topOffset-4);
				g.drawString(sNeg,xUserToScreen(-currentX)+leftOffset-
										fm.stringWidth(sNeg)/2,
										yUserToScreen(0.0)+3+topOffset+fm.getHeight());
				currentX += xTickScale;
			}
			double currentY = yTickScale;
			double maxY = Math.max(Math.abs(ymin),Math.abs(ymax));
			while(currentY < maxY)
			{
				sPos = numToString(currentY,1);
				sNeg = numToString(-currentY,1);
				g.drawLine(xUserToScreen(0.0)+leftOffset+5,
							  yUserToScreen(currentY)+topOffset,
							  xUserToScreen(0.0)+leftOffset-4,
							  yUserToScreen(currentY)+topOffset);
				g.drawString(sPos,xUserToScreen(0.0)+leftOffset-
										5-fm.stringWidth(sPos),
										yUserToScreen(currentY)+topOffset+fm.getHeight()/2-2);
				g.drawLine(xUserToScreen(0.0)+leftOffset+5,
							  yUserToScreen(-currentY)+topOffset,
							  xUserToScreen(0.0)+leftOffset-4,
							  yUserToScreen(-currentY)+topOffset);
				g.drawString(sNeg,xUserToScreen(0.0)+leftOffset-
										5-fm.stringWidth(sNeg),
										yUserToScreen(-currentY)+topOffset+fm.getHeight()/2-2);
				currentY += yTickScale;
			}
		}
		else if(axisType == PERIMETER)
		{
		//
		// Draw tick marks starting at (xmin,ymin) and moving outward
		//
			double currentX = xmin+xTickScale;
			String s;
			while(currentX < xmax)
			{
				s = numToString(currentX,0);
				g.drawLine(xUserToScreen(currentX)+leftOffset,
							  getSize().height-bottomOffset+5,
						  	  xUserToScreen(currentX)+leftOffset,
							  getSize().height-bottomOffset-4);
				g.drawString(s,xUserToScreen(currentX)+leftOffset-fm.stringWidth(s)/2,
									getSize().height-bottomOffset+fm.getHeight()+3);
				currentX += xTickScale;
			}
			double currentY = ymin+yTickScale;
			while(currentY < ymax)
			{
				s = numToString(currentY,1);
				g.drawLine(leftOffset-4,yUserToScreen(currentY)+topOffset,
							  leftOffset+5,yUserToScreen(currentY)+topOffset);
				g.drawString(s,leftOffset-fm.stringWidth(s)-4,
									yUserToScreen(currentY)+topOffset+fm.getHeight()/2-2);
				currentY += yTickScale;
			}
		}
		else if(axisType == XAXIS)
		{
	    //
		// Draw tick marks starting at (xmin,ymin) and moving outward
		//
			double currentX = xmin+xTickScale;
			String s; String sPos; String sNeg;
			while(currentX < xmax)
			{
		        s = numToString(currentX,0);
				g.drawLine(xUserToScreen(currentX)+leftOffset,
							  yUserToScreen(0.0)+5+topOffset,
							  xUserToScreen(currentX)+leftOffset,
							  yUserToScreen(0.0)+topOffset-4);
				g.drawString(s,xUserToScreen(currentX)+leftOffset-
										fm.stringWidth(s)/2,
										yUserToScreen(0.0)+3+topOffset+fm.getHeight());
			    currentX += xTickScale;
			} 
					        
		    double currentY = ymin+yTickScale;
		    if((xmin >= 0)||(xmax <= 0))
		    {
			while(currentY < ymax)
			{
				s = numToString(currentY,1);
				g.drawLine(leftOffset-4,yUserToScreen(currentY)+topOffset,
							  leftOffset+5,yUserToScreen(currentY)+topOffset);
				g.drawString(s,leftOffset+4,
									yUserToScreen(currentY)+topOffset+fm.getHeight()/2-2);
				currentY += yTickScale;
			}}
			else
			{
			currentY = yTickScale;
			double maxY = Math.max(Math.abs(ymin),Math.abs(ymax));
			while(currentY < maxY)
			{
				sPos = numToString(currentY,1);
				sNeg = numToString(-currentY,1);
				g.drawLine(xUserToScreen(0.0)+leftOffset+5,
							  yUserToScreen(currentY)+topOffset,
							  xUserToScreen(0.0)+leftOffset-4,
							  yUserToScreen(currentY)+topOffset);
				g.drawString(sPos,xUserToScreen(0.0)+leftOffset-
										5-fm.stringWidth(sPos),
										yUserToScreen(currentY)+topOffset+fm.getHeight()/2-2);
				g.drawLine(xUserToScreen(0.0)+leftOffset+5,
							  yUserToScreen(-currentY)+topOffset,
							  xUserToScreen(0.0)+leftOffset-4,
							  yUserToScreen(-currentY)+topOffset);
				g.drawString(sNeg,xUserToScreen(0.0)+leftOffset-
										5-fm.stringWidth(sNeg),
										yUserToScreen(-currentY)+topOffset+fm.getHeight()/2-2);
				currentY += yTickScale;
			}
			}
		}
	}
	private void paintTitle()
	{
		if(title == null) return;
		if(pGraphics == null) return;
		
		Graphics g = pGraphics;
		g.setColor(titleColor);
		g.setFont(titleFont);
		FontMetrics fm = g.getFontMetrics(titleFont);
		g.drawString(title,getSize().width/2-fm.stringWidth(title)/2,fm.getHeight());
	}
private void paintAxes()
{
	    if(pGraphics == null) return;
	    
	    Graphics g = pGraphics;
		g.setColor(Color.black);
		switch(axisType)
		{
			case NONE:

				return;

			case PERIMETER:

				g.drawRect(leftOffset,topOffset,1,
							  getSize().height-bottomOffset-topOffset);
				g.drawRect(leftOffset,getSize().height-bottomOffset,
							  getSize().width-leftOffset-rightOffset,1);
				break;

			case FLOATING:
			
			    g.clipRect(leftOffset,topOffset,getSize().width-leftOffset-rightOffset,
					  getSize().height-topOffset-bottomOffset);
				g.drawRect(xUserToScreen(0.0)+leftOffset,topOffset,1,
							  getSize().height-topOffset-bottomOffset);
				g.drawRect(leftOffset,yUserToScreen(0.0)+topOffset,
							  getSize().width-leftOffset-rightOffset,1);
				break;
				
		    case XAXIS:
		    			
			    g.clipRect(leftOffset,topOffset,getSize().width-leftOffset-rightOffset,
					  getSize().height-topOffset-bottomOffset);
			    							  
				if((xmin >= 0.0)||(xmax <= 0.0))
				{
				g.drawRect(leftOffset,topOffset,1,                       // left edge
							  getSize().height-bottomOffset-topOffset);
			    }
			    else
			    {
			    g.drawRect(xUserToScreen(0.0)+leftOffset,topOffset,1,
							  getSize().height-topOffset-bottomOffset);
			    }

				g.drawRect(leftOffset,yUserToScreen(0.0)+topOffset,
							  getSize().width-leftOffset-rightOffset,1);
				break;  
		}
	}
//
// UTILITY METHODS
//
private void setDefaults()
	{
		xmin = 1.0;
		ymin = 1.0;
		xmax = 1.0;
		ymax = 1.0;
		data = new Vector();
		autoScale = true;
		autoTickScaling = true;
		axisType = PERIMETER;
		leftOffset = 0;
		rightOffset = 0;
		topOffset = 0;
		bottomOffset = 0;
		drawAxisLabels = false;
		axisLabelFont = new Font("TimesNewRoman",Font.PLAIN,12);
		titleFont = new Font("TimesNewRoman",Font.PLAIN,12);
		drawBoundPoints = false;
		boundPointsFont = new Font("TimesNewRoman",Font.PLAIN,12);
		boundPointsColor = Color.black;
		drawGridLines = false;
		drawTicks = true;
		drawZoom = true;
		firstX = 0;
		firstY = 0;
		lastX = 0;
		lastY = 0;
		xTickScale = 0;
		yTickScale = 0;
		NtoS = new NumberToString();
		xPrecision = 2;
		yPrecision = 2;
		xLabelScientific = true;
		yLabelScientific = true;
		yMaxDefault =  20.0;
		yMinDefault = -20.0;
	}
//
// Converts a number to a String for tick mark labels
// to be implemented later:  different types, floating point
// scientific, or fixed point
//
private String numToString(double d, int type)
	{
	    if(type == 0) 
	    {
	        NtoS.setPrecision(xPrecision);
	        NtoS.setScientific(xLabelScientific);
	    }
	    else if (type == 1) 
	    {
	        NtoS.setPrecision(yPrecision);
	        NtoS.setScientific(yLabelScientific);
	    }
		return new String(NtoS.toString(d));
	}
//
// Finds the PlotData given by name, returns null if not found
//
private PlotData findPlotData(String name)
	{
		// Find the plot
		int end = data.size();
		for(int i=0;i<end;i++)
		{
			PlotData pData = (PlotData)data.elementAt(i);
			if(pData.isPlotName(name))
			{
				return pData;
			}
		}
		return null;		
	}
//
// Converts user coordinates to screen coordinates taking
// the offsets in account
//
private int xUserToScreen(double x)
{
		return (int)((x-xmin)/Math.abs(xmax-xmin)*
			(double)(getSize().width-leftOffset-rightOffset));
}
private int yUserToScreen(double y)
{
		return (int)((ymax-y)/Math.abs(ymax-ymin)*
			(double)(getSize().height-topOffset-bottomOffset));
}
//
// Converts screen coordinates to user coordinates
//
private double xScreenToUser(int x)
{
		return xmin+(double)(x-leftOffset)*Math.abs(xmax-xmin)/
				 (double)(getSize().width-leftOffset-rightOffset);
}
private double yScreenToUser(int y)
{
		return ymax-(double)(y-topOffset)*Math.abs(ymax-ymin)/
				 (double)(getSize().height-topOffset-bottomOffset);
}
//
// Sets the left, right, top and bottom offsets
// taking various elements into account, like title height
//
	private void setOffsets()
	{
		topOffset    = 5;
		bottomOffset = 5;
		leftOffset   = 5; 
		rightOffset  = 5;
		Graphics g = getGraphics();
		FontMetrics fmBoundary =
		g.getFontMetrics(boundPointsFont);
		FontMetrics fmTitle = g.getFontMetrics(titleFont);
		FontMetrics fmTickLabels =
		g.getFontMetrics(new Font("TimesNewRoman",Font.PLAIN,10));
		FontMetrics fmAxisLabels = g.getFontMetrics(axisLabelFont);
	//
	// Set the topOffset taking the title height and boundary
	// point height into account
	//
		if(drawBoundPoints && title != null)
			topOffset += Math.max(fmBoundary.getHeight(),fmTitle.getHeight());
		else if(drawBoundPoints)
			topOffset += fmBoundary.getHeight();
		else if(title != null)
			topOffset += fmTitle.getHeight();
	
	//
	// Set the bottomOffset taking the axisLabel, 
	// boundary point, axisType, tickMark labels
	// into account
	//
		if(drawTicks && axisType == PERIMETER)
			bottomOffset += fmTickLabels.getHeight();
		if(drawBoundPoints && drawAxisLabels)
			bottomOffset += Math.max(fmBoundary.getHeight(),
											 fmAxisLabels.getHeight())+5;
		else if(drawBoundPoints)
			bottomOffset += fmBoundary.getHeight()+5;
		else if(drawAxisLabels)
			bottomOffset += fmAxisLabels.getHeight()+5;
			
	//
	// Set the leftOffset taking the axisLabel, axisType, and 
	// tickMark labels into account
	//
		if(drawTicks && axisType == PERIMETER)
			leftOffset += fmTickLabels.stringWidth("-0.000000");
		if(drawAxisLabels)
			leftOffset += fmAxisLabels.stringWidth("G")+10;
	}
//
// Mouse HANDLING METHODS
//
	class SymMouse extends java.awt.event.MouseAdapter
	{
		public void mousePressed(java.awt.event.MouseEvent event)
		{
				mousePressedInPlot(event.getX(),event.getY());
		}

	}
	class SymMouseMotion extends java.awt.event.MouseMotionAdapter
	{
		public void mouseDragged(java.awt.event.MouseEvent event)
		{
				mouseDraggedInPlot(event.getX(),event.getY());
		}
	}
//
// Handle the drawing of the zoom rectangle
//
private void mousePressedInPlot(int x, int y)
{
		if(!drawZoom)return;

		Graphics g = getGraphics();
		g.setXORMode(new Color(96,60,242));
	//
	// Draw over last drawn rectangle
	//
		g.drawLine(firstX,firstY,firstX,lastY);
		g.drawLine(firstX,firstY,lastX,firstY);
		g.drawLine(lastX,firstY,lastX,lastY);
		g.drawLine(firstX,lastY,lastX,lastY);
	//
	// Bail out if not in the drawing area
	//
		if(x<leftOffset || x>getSize().width-rightOffset ||
			y<topOffset || y>getSize().height-bottomOffset)
		{
			firstX = lastX = firstY = lastY = 0;
			return;
		}

		firstX = x;
		firstY = y;
		lastX = x;
		lastY = y;

		return;
	}
//
// Handle the drawing of the zoom rectangle
//
private void mouseDraggedInPlot(int x, int y)
{
	//
	// If first point is at origin, bail out
	//
		if(firstX == 0 && firstY == 0) return ;

		Graphics g = getGraphics();
		g.setXORMode(new Color(96,60,242));

	//
	// Draw over last drawn rectangle
	//
		g.drawLine(firstX,firstY,firstX,lastY);
		g.drawLine(firstX,firstY,lastX,firstY);
		g.drawLine(lastX,firstY,lastX,lastY);
		g.drawLine(firstX,lastY,lastX,lastY);
		
	//
	// Make sure mouse is in drawing area
	//
		if(x>leftOffset && x<getSize().width-rightOffset &&
			y>topOffset && y<getSize().height-bottomOffset)
			{
				lastY = y;
				lastX = x;
			}
		else if((x<leftOffset || x>getSize().width-rightOffset) &&
				  y>topOffset && y<getSize().height-bottomOffset)
			lastY = y;
		else if((y<topOffset || y>getSize().height-bottomOffset) &&
				  x>leftOffset && x<getSize().width-rightOffset)
			lastX = x;
		else
		{
			// do nothing
		}
	//
	// Draw new rectangle
	//
		g.drawLine(firstX,firstY,firstX,lastY);
		g.drawLine(firstX,firstY,lastX,firstY);
		g.drawLine(lastX,firstY,lastX,lastY);
		g.drawLine(firstX,lastY,lastX,lastY);
   //
   //  Update new Zoom Limits
   //
   		ZoomXmin = xScreenToUser(Math.min(firstX, lastX));
		ZoomYmin = yScreenToUser(Math.max(firstY, lastY));
		ZoomXmax = xScreenToUser(Math.max(firstX, lastX));
		ZoomYmax = yScreenToUser(Math.min(firstY, lastY));
	//
	//  Clean up
	//
		g.dispose();
		return;
}
/**
Returns minimum X coordinate of zoom box. (In user coordinates.)
*/
public double getZoomXmin(){ return ZoomXmin;}
/**
Returns maximum X coordinate of zoom box. (In user coordinates.)
*/
public double getZoomXmax(){ return ZoomXmax;}
/**
Returns minimum Y coordinate of zoom box. (In user coordinates.)
*/
public double getZoomYmin(){ return ZoomYmin;}
/**
Returns maximum Y coordinate of zoom box. (In user coordinates.)
*/
public double getZoomYmax(){ return ZoomYmax;}

//***************************************************************
//                           DATA
//***************************************************************
//
// The bounding rectangle
//
	private double xmin, ymin, xmax, ymax;
//
// Information for handling and drawing the zoom rectangle
//
	private int firstX, firstY, lastX, lastY;
//
// The boundary around the plot
//
	private int leftOffset, rightOffset, topOffset, bottomOffset;
//
// Plotting data: x and y coordinate arrays, a color, a type,
// a character representing the point type, and a name, all 
// stored in a Vector
//
	private Vector data;
//
// The axes type
//
	private int axisType;
	/**
	Axis Type : no axis plotted
	*/
	public static final int NONE      = 0;
	/**
	Axis Type : perimeter axis plotted
	*/
	public static final int PERIMETER = 1;
	/**
	Axis Type : floating axis plotted
	*/
	public static final int FLOATING  = 2;
	/**
	Axis Type : only x-axis plotted
	*/
	public static final int XAXIS     = 3;
//
// Plot Type
//
    /**
    Plot Type : data points connected with line segments
    */
  	public static final int CURVE = 0;
  	/**
    Plot Type : data points indicated with character marker.
  	*/
	public static final int POINTS = 1;
	/**
    Plot Type : data points connected with line segments, and data points
    indicated with character marker.
  	*/
	public static final int CURVE_AND_POINTS = 2;
//
//
// FLAGS
//
	private boolean drawZoom;			// Draw zoom rectangle
	private boolean drawAxisLabels;	// Draw axis labels
	private boolean autoScale;		// Automatically generate bounding box
	private boolean autoTickScaling;	// Automatically generate tick scales
	private boolean drawTicks;		// Draw tick marks and labels
	private boolean drawBoundPoints;	// Draw boundary points in corners
	private boolean drawGridLines;	// Draw grid lines
//
// The distance between tick marks given in user coordinates
//
	private double xTickScale;
	private double yTickScale;
//
// The plot title and information
//
	private String title;
	private Font titleFont;
	private Color titleColor;
//
// Axis label strings and information
//
	private String yAxisLabel;
	private String xAxisLabel;
	private Font axisLabelFont;
	private Color axisLabelColor;
	
//
//  Axis label precision and type
//
    /** 
    Label Format : scientific 
    */
	public static final int SCIENTIFIC = 0;
	/**
	Label Format : fixed point 
	*/
	public static final int FIXED = 1;
	
	private int xPrecision = 2;
	private int yPrecision = 2;
	
	private boolean xLabelScientific;
	private boolean yLabelScientific;
	
	private double yMaxDefault;
	private double yMinDefault;
//
// Boundary point drawing information
//
	private Font boundPointsFont;
	private Color boundPointsColor;
//
//  Number to String Conversion object
//
    NumberToString NtoS;
//
//  Zoom Limits
//
    double ZoomXmin;
    double ZoomYmin;
    double ZoomXmax;
    double ZoomYmax;
//
//  Local graphics reference
//
    Graphics pGraphics = null;
}