package cam.visuals.plots;
//***************************************************************
//				File:  PlotFrame.java
//				Created by Mark Hoefer
//
//              Modifications by Chris Anderson 
//              (Java 1.1 migration, bug fixes, added printing)
//				(C) UCLA 1997, 1998
//***************************************************************
//

import java.awt.*;
import java.util.Vector;
/**
A frame which plots x and y data and has controls for viewing
the plot. The plotting functionality is provided by an internal 
instance of the Plot class.  In addition to the output control provided
by the user interface, one can set plot formatting by invoking methods 
of the internal Plot instance. For example if PF is a PlotFrame instance, then
the code
<pre>
    Plot P = PF.getPlot();
    P.setXlabelFormat(Plot.FIXED);
    P.setXlabelPrecision(3);
    P.setYlabelFormat(Plot.SCIENTIFIC);
    P.setYlabelPrecision(2);
</pre>
sets the X label format to fixed output and 3 decimal digits and the Y label format
to scientific  with 2 decimal digits.
@author Mark Hoefer, Chris Anderson
@version Oct. 28, 1998
*/
public class PlotFrame extends Frame
{
//***************************************************************
//                        DATA
//***************************************************************
//
// The canvas of the PlotFrame where all the drawing is done
//
	protected Plot plotCanvas;
//
// Colors to be used for plots
//
	private Color[] colors = 
	{Color.blue,   Color.red,  Color.green, Color.orange,
	Color.magenta, Color.pink, Color.cyan,  Color.yellow,
	Color.gray,    Color.black};
	
	private static final int numColors = 10;
//
// Number of plots, to be used for coloring purposes
//
	private int numPlots;
//
// The current viewing rectangle
//
	protected double currentXMin = 0,
						  currentYMin = 0,
						  currentXMax = 0,
						  currentYMax = 0;
//
// The original bounding rectangle
//
	protected double xmin = 0,
						  ymin = 0,
						  xmax = 0,
						  ymax = 0;
//
// The label precision
//
	private int xPrecision = 2,
	              yPrecision = 2;
//
// The Legend, needed so it can reflect updates
//
	protected LegendDialog legend;
//
// Title information
//
	String title = null;
	Font titleFont;
	Color titleColor;

	public PlotFrame()
	{
	    //{{INIT_CONTROLS
		setLayout(new BorderLayout(0,0));
		setVisible(false);
		setSize(537,349);
		ControlPanel = new java.awt.Panel();
		ControlPanel.setLayout(new FlowLayout(FlowLayout.CENTER,10,25));
		ControlPanel.setBounds(0,323,736,74);
		add("South", ControlPanel);
		ZoomOutButton = new java.awt.Button();
		ZoomOutButton.setLabel("Zoom Out: 2x");
		ZoomOutButton.setBounds(197,25,82,24);
		ZoomOutButton.setFont(new Font("TimesRoman", Font.PLAIN, 12));
		ControlPanel.add(ZoomOutButton);
		ZoomInButton = new java.awt.Button();
		ZoomInButton.setLabel("Zoom In: 2x");
		ZoomInButton.setBounds(289,25,73,24);
		ZoomInButton.setFont(new Font("TimesRoman", Font.PLAIN, 12));
		ControlPanel.add(ZoomInButton);
		ZoomBoxButton = new java.awt.Button();
		ZoomBoxButton.setLabel("Zoom Box");
		ZoomBoxButton.setBounds(372,25,65,24);
		ZoomBoxButton.setFont(new Font("TimesRoman", Font.PLAIN, 12));
		ControlPanel.add(ZoomBoxButton);
		ResetButton = new java.awt.Button();
		ResetButton.setLabel("Reset");
		ResetButton.setBounds(447,25,41,24);
		ResetButton.setFont(new Font("TimesRoman", Font.PLAIN, 12));
		ControlPanel.add(ResetButton);
		CloseButton = new java.awt.Button();
		CloseButton.setLabel("Close");
		CloseButton.setBounds(498,25,41,24);
		CloseButton.setFont(new Font("TimesRoman", Font.PLAIN, 12));
		ControlPanel.add(CloseButton);
		//}}

		//{{INIT_MENUS
		MainMenu = new java.awt.MenuBar();
		FileMenu = new java.awt.Menu("File");
		PrintPlot = new java.awt.MenuItem("Print");
		FileMenu.add(PrintPlot);
		MainMenu.add(FileMenu);
		LegendMenu = new java.awt.Menu("Legend");
		PlotLegend_mi = new java.awt.MenuItem("Plot Legend");
		LegendMenu.add(PlotLegend_mi);
		MainMenu.add(LegendMenu);
		ViewMenu = new java.awt.Menu("View");
		AxisMenu = new java.awt.Menu("Set Axes");
		AxisPerimeter_mi = new java.awt.MenuItem("Perimeter");
		AxisMenu.add(AxisPerimeter_mi);
		AxisFloating_mi = new java.awt.MenuItem("Floating");
		AxisMenu.add(AxisFloating_mi);
		AxisNone_mi = new java.awt.MenuItem("None");
		AxisMenu.add(AxisNone_mi);
		ViewMenu.add(AxisMenu);
		GridMenu = new java.awt.Menu("Gridlines");
		GridOn_mi = new java.awt.MenuItem("On");
		GridMenu.add(GridOn_mi);
		GridOff_mi = new java.awt.MenuItem("Off");
		GridMenu.add(GridOff_mi);
		ViewMenu.add(GridMenu);
		LabelMenu = new java.awt.Menu("Labels");
		LabelShowOn_mi = new java.awt.MenuItem("Show");
		LabelMenu.add(LabelShowOn_mi);
		LabelShowOff_mi = new java.awt.MenuItem("Do Not Show");
		LabelMenu.add(LabelShowOff_mi);
		ViewMenu.add(LabelMenu);
		TickMark_mi = new java.awt.MenuItem("Set Tick Mark Scale ...");
		ViewMenu.add(TickMark_mi);
		Bounding_mi = new java.awt.MenuItem("Set Bounding Rectangle ...");
		ViewMenu.add(Bounding_mi);
		MainMenu.add(ViewMenu);
		CharacteristicsMenu = new java.awt.Menu("Plot Characteristics");
		MainMenu.add(CharacteristicsMenu);
		setMenuBar(MainMenu);
		//$$ MainMenu.move(0,0);
		//}}
		
		//{{REGISTER_LISTENERS
		SymWindow aSymWindow = new SymWindow();
		this.addWindowListener(aSymWindow);
		SymAction lSymAction = new SymAction();
		ZoomOutButton.addActionListener(lSymAction);
		ZoomInButton.addActionListener(lSymAction);
		ZoomBoxButton.addActionListener(lSymAction);
		ResetButton.addActionListener(lSymAction);
		CloseButton.addActionListener(lSymAction);
		AxisPerimeter_mi.addActionListener(lSymAction);
		AxisFloating_mi.addActionListener(lSymAction);
		AxisNone_mi.addActionListener(lSymAction);
		GridOn_mi.addActionListener(lSymAction);
		GridOff_mi.addActionListener(lSymAction);
		LabelShowOn_mi.addActionListener(lSymAction);
		LabelShowOff_mi.addActionListener(lSymAction);
		PlotLegend_mi.addActionListener(lSymAction);
		TickMark_mi.addActionListener(lSymAction);
		Bounding_mi.addActionListener(lSymAction);
		PrintPlot.addActionListener(lSymAction);
		//}}
		
		setTitle("Plot Display");
		plotCanvas = new Plot();
		add("Center",plotCanvas);


		// instantiate legend so that it is non-null if PlotInfoDialog calls it
		legend = new LegendDialog(this);
		numPlots = 0;
		
	    PlotCharAction = new PlotPropertiesAction();	
	}
	
	//{{DECLARE_CONTROLS
	java.awt.Panel ControlPanel;
	java.awt.Button ZoomOutButton;
	java.awt.Button ZoomInButton;
	java.awt.Button ZoomBoxButton;
	java.awt.Button ResetButton;
	java.awt.Button CloseButton;
	//}}

	//{{DECLARE_MENUS
	java.awt.MenuBar MainMenu;
	java.awt.Menu FileMenu;
	java.awt.MenuItem PrintPlot;
	java.awt.Menu LegendMenu;
	java.awt.MenuItem PlotLegend_mi;
	java.awt.Menu ViewMenu;
	java.awt.Menu AxisMenu;
	java.awt.MenuItem AxisPerimeter_mi;
	java.awt.MenuItem AxisFloating_mi;
	java.awt.MenuItem AxisNone_mi;
	java.awt.Menu GridMenu;
	java.awt.MenuItem GridOn_mi;
	java.awt.MenuItem GridOff_mi;
	java.awt.Menu LabelMenu;
	java.awt.MenuItem LabelShowOn_mi;
	java.awt.MenuItem LabelShowOff_mi;
	java.awt.MenuItem TickMark_mi;
	java.awt.MenuItem Bounding_mi;
	java.awt.Menu CharacteristicsMenu;
	//}}
	
	protected PlotPropertiesAction PlotCharAction;
	
	
	public PlotFrame(String title)
	{
		this();
		setTitle(title);
	}
	
	public void show()
    {
		reshape(210,50,400,400);
		super.show();
    }
    
	public void addNotify()
	{
  	    // Record the size of the window prior to calling parents addNotify.
	    Dimension d = getSize();

		super.addNotify();

		if (fComponentsAdjusted)
			return;

		// Adjust components according to the insets
		setSize(insets().left + insets().right + d.width, insets().top + insets().bottom + d.height);
		Component components[] = getComponents();
		for (int i = 0; i < components.length; i++)
		{
			Point p = components[i].getLocation();
			p.translate(insets().left, insets().top);
			components[i].setLocation(p);
		}
		fComponentsAdjusted = true;
	}

    // Used for addNotify check.
	boolean fComponentsAdjusted = false;
//
//#####################################################################    
//
//#####################################################################
//
/**
Sets the title of the plot
*/
public void setPlotTitle(String title, Font f, Color c)
{
		plotCanvas.setTitle(title,f,c);
		titleFont  = f;
		titleColor = c;
}
/**
Sets the axis labels (or titles).
*/
public void setAxisLabels(String xLabel, String yLabel, Font f, Color c)
{
		plotCanvas.setAxisLabels(xLabel,yLabel,f,c);
}

/**
Sets the x and y axis limits for the plot.  
*/
public void setXYBounds(double xmin, double xmax,
								 double ymin, double ymax)
{
		plotCanvas.setXYBounds(xmin,xmax,ymin,ymax);
		currentXMin = xmin;
		currentYMin = ymin;
		currentXMax = xmax;
		currentYMax = ymax;
}
/**
Sets the x axis limits for the plot.
*/
public void setXbounds(double xmin, double xmax)
{
		plotCanvas.setXbounds(xmin,xmax);
}
//
// PLOTTING METHODS
//
/**
Plots the input data in the current plot and associates a name with the plot.
@param xdata double array containing the x-coordinates for the plot
@param ydata double array containing the y-coordinates for the plot
@param plotName string name of the plot
*/
public void addPlot(double[] xdata, double[] ydata, String plotName)
{
	//
	// Add the new plot to the menu if not already added
	//
		int num = CharacteristicsMenu.countItems();
		boolean isPlotPresent = false;
		for(int i=0;i<num;i++)
			if(CharacteristicsMenu.getItem(i).getLabel().equals(plotName))
			{
				isPlotPresent = true;
				break;
			}
		int index;
		if(!isPlotPresent)
		{
		    MenuItem newItem = new MenuItem(plotName);
		    newItem.addActionListener(PlotCharAction);
			CharacteristicsMenu.add(newItem);
			numPlots++;
		}
		index = numPlots%numColors;
		plotCanvas.plot(xdata,ydata,colors[index], plotName);

	//
	// Set all the maxes and mins
	//
		xmin = plotCanvas.getXMin();
		ymin = plotCanvas.getYMin();
		xmax = plotCanvas.getXMax();
		ymax = plotCanvas.getYMax();
		currentXMin = xmin;
		currentYMin = ymin;
		currentXMax = xmax;
		currentYMax = ymax;
	}
/**
Plots the input data in the current plot and associates a name with the plot.
@param xdata Vector of Doubles containing the x-coordinates for the plot
@param ydata Vector of Doubles containing the y-coordinates for the plot
@param plotName string name of the plot
*/
	public void addPlot(Vector xdata, Vector ydata, String plotName)
	{
	//
	// Make sure both Vectors have same length
	//
		if(xdata.size() != ydata.size())
		{
			System.err.println("In PlotFrame.addPlot: Vectors not of same size");
			return;
		}
		double[] x = new double[xdata.size()];
		double[] y = new double[ydata.size()];

		for(int i=0;i<xdata.size();i++)
		{
			x[i] = ((Double)xdata.elementAt(i)).doubleValue();
			y[i] = ((Double)ydata.elementAt(i)).doubleValue();
		}
		addPlot(x,y,plotName);
	}
//
// Removes all plots
/**
Remotes all plots
*/
public void clearPlots()
{
		plotCanvas.clear();
		numPlots = 0;
		for(int i=0;i<CharacteristicsMenu.countItems();i++)
			CharacteristicsMenu.remove(i);
}
/**
Returns the contained Plot instance.
*/
public Plot getPlot()
{
    return plotCanvas;
}
//
// Zooms in by the given percents.  If >100 then zoom out, <100 zoom in
//
protected void zoom(double xPercent, double yPercent)
{
		plotCanvas.zoom(xPercent,yPercent);
		currentXMin = plotCanvas.getXMin();
		currentYMin = plotCanvas.getYMin();
		currentXMax = plotCanvas.getXMax();
		currentYMax = plotCanvas.getYMax();
}


//
//#####################################################################    
//
//#####################################################################
//
//
// EVENT HANDLING METHODS
//

	class SymWindow extends java.awt.event.WindowAdapter
	{
		public void windowClosing(java.awt.event.WindowEvent event)
		{
			Object object = event.getSource();
			if (object == PlotFrame.this)
				PlotFrame_WindowClosing(event);
		}
	}
	
	void PlotFrame_WindowClosing(java.awt.event.WindowEvent event)
	{
		setVisible(false);	// hide the Frame
	}

	class PlotPropertiesAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
            new PlotInfoDialog(PlotFrame.this, ((MenuItem)object).getLabel());
        }
    }
    
	class SymAction implements java.awt.event.ActionListener
	{
		public void actionPerformed(java.awt.event.ActionEvent event)
		{
			Object object = event.getSource();
			if (object == ZoomOutButton)
				ZoomOutButton_ActionPerformed(event);
			else if (object == ZoomInButton)
				ZoomInButton_ActionPerformed(event);
			else if (object == ZoomBoxButton)
				ZoomBoxButton_ActionPerformed(event);
			else if (object == ResetButton)
				ResetButton_ActionPerformed(event);
			else if (object == CloseButton)
				CloseButton_ActionPerformed(event);
			else if (object == AxisPerimeter_mi)
				AxisPerimeterMi_ActionPerformed(event);
			else if (object == AxisFloating_mi)
				AxisFloatingMi_ActionPerformed(event);
			else if (object == AxisNone_mi)
				AxisNoneMi_ActionPerformed(event);
			else if (object == GridOn_mi)
				GridOnMi_ActionPerformed(event);
			else if (object == GridOff_mi)
				GridOffMi_ActionPerformed(event);
			else if (object == LabelShowOn_mi)
				LabelShowOnMi_ActionPerformed(event);
			else if (object == LabelShowOff_mi)
				LabelShowOffMi_ActionPerformed(event);
			else if (object == PlotLegend_mi)
				PlotLegendMi_ActionPerformed(event);
			else if (object == TickMark_mi)
				TickMarkMi_ActionPerformed(event);
			else if (object == Bounding_mi)
				BoundingMi_ActionPerformed(event);
			else if (object == PrintPlot)
				PrintPlot_ActionPerformed(event);
			
		}
	}

	void ZoomOutButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
		zoom(200.0,200.0);
		plotCanvas.validate();
	    plotCanvas.repaint();
	}

	void ZoomInButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
		zoom(50.0,50.0);
		plotCanvas.validate();
	    plotCanvas.repaint();
	}

	void ZoomBoxButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
	double zoomXMin = plotCanvas.getZoomXmin();
	double zoomXMax = plotCanvas.getZoomXmax();
	double zoomYMin = plotCanvas.getZoomYmin();
	double zoomYMax = plotCanvas.getZoomYmax();
    if(zoomXMin == zoomXMax || zoomYMin == zoomYMax) return;
    setXYBounds(zoomXMin,zoomXMax,zoomYMin,zoomYMax);
   	plotCanvas.validate();
	plotCanvas.repaint();
	}

	void ResetButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
        setXYBounds(xmin,xmax,ymin,ymax);
        plotCanvas.setAutoTickScaling(true);
        plotCanvas.validate();
        plotCanvas.repaint();
	}

	void CloseButton_ActionPerformed(java.awt.event.ActionEvent event)
	{
        legend.hide();
        setVisible(false);
	}


	void AxisPerimeterMi_ActionPerformed(java.awt.event.ActionEvent event)
	{
		plotCanvas.setAxisType(Plot.PERIMETER);
		plotCanvas.validate();
		plotCanvas.repaint();
	}

	void AxisFloatingMi_ActionPerformed(java.awt.event.ActionEvent event)
	{
		plotCanvas.setAxisType(Plot.FLOATING);
	    plotCanvas.validate();
		plotCanvas.repaint();
	}

	void AxisNoneMi_ActionPerformed(java.awt.event.ActionEvent event)
	{
		plotCanvas.setAxisType(Plot.NONE);
	    plotCanvas.validate();
		plotCanvas.repaint();
	}

	void GridOnMi_ActionPerformed(java.awt.event.ActionEvent event)
	{
		plotCanvas.setGridLines(true);
	    plotCanvas.validate();
		plotCanvas.repaint();
	}

	void GridOffMi_ActionPerformed(java.awt.event.ActionEvent event)
	{
        plotCanvas.setGridLines(false);
        plotCanvas.validate();
		plotCanvas.repaint();
	}

	void LabelShowOnMi_ActionPerformed(java.awt.event.ActionEvent event)
	{
        plotCanvas.setDrawBoundPoints(true);
        plotCanvas.setDrawAxisLabels(true);
        plotCanvas.setTitle(title,titleFont,titleColor);
        plotCanvas.validate();
		plotCanvas.repaint();
	}

	void LabelShowOffMi_ActionPerformed(java.awt.event.ActionEvent event)
	{
        plotCanvas.setDrawBoundPoints(false);
        plotCanvas.setDrawAxisLabels(false);
        plotCanvas.removeTitle();
        plotCanvas.validate();
		plotCanvas.repaint();
	}

	void PlotLegendMi_ActionPerformed(java.awt.event.ActionEvent event)
	{
        legend = new LegendDialog(this);
        legend.setVisible(true);
	}

	void TickMarkMi_ActionPerformed(java.awt.event.ActionEvent event)
	{
	    GetTickDialog Td = new GetTickDialog(this);
	    Td.setVisible(true);
		
	}

	void BoundingMi_ActionPerformed(java.awt.event.ActionEvent event)
	{
        GetBoundsDialog Bd = new GetBoundsDialog(this);
        Bd.setVisible(true);
	}

	void PrintPlot_ActionPerformed(java.awt.event.ActionEvent event)
	{
		Toolkit toolkit = this.getToolkit();
		PrintJob job    = toolkit.getPrintJob(this,"A Plot",printprefs);
		if(job == null) return;
		Graphics page      = job.getGraphics();
		Dimension size     = plotCanvas.getSize();
		Dimension pageSize = job.getPageDimension();
		page.translate((pageSize.width - size.width)/2, (pageSize.height - size.height)/2);
		page.drawRect(-1,-2,size.width+1, size.height+1);
		page.setClip(0,0,size.width, size.height);
		plotCanvas.paint(page);
		page.dispose();
		job.end();
	}
	// preferences for printing ....
	private static java.util.Properties printprefs = new java.util.Properties();
}